/*******************************************************************************
* Copyright (C) 2021 Intel Corporation
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

/*
!  Content:
!      Intel(R) oneAPI Math Kernel Library (oneMKL) include for transposition routines
!******************************************************************************/

#define mkl_simatcopy MKL_Simatcopy
#define mkl_dimatcopy MKL_Dimatcopy
#define mkl_cimatcopy MKL_Cimatcopy
#define mkl_zimatcopy MKL_Zimatcopy
#define mkl_somatcopy MKL_Somatcopy
#define mkl_domatcopy MKL_Domatcopy
#define mkl_comatcopy MKL_Comatcopy
#define mkl_zomatcopy MKL_Zomatcopy
#define mkl_somatcopy2 MKL_Somatcopy2
#define mkl_domatcopy2 MKL_Domatcopy2
#define mkl_comatcopy2 MKL_Comatcopy2
#define mkl_zomatcopy2 MKL_Zomatcopy2
#define mkl_somatadd MKL_Somatadd
#define mkl_domatadd MKL_Domatadd
#define mkl_comatadd MKL_Comatadd
#define mkl_zomatadd MKL_Zomatadd
#define mkl_simatcopy_batch_strided MKL_Simatcopy_batch_strided
#define mkl_dimatcopy_batch_strided MKL_Dimatcopy_batch_strided
#define mkl_cimatcopy_batch_strided MKL_Cimatcopy_batch_strided
#define mkl_zimatcopy_batch_strided MKL_Zimatcopy_batch_strided
#define mkl_somatcopy_batch_strided MKL_Somatcopy_batch_strided
#define mkl_domatcopy_batch_strided MKL_Domatcopy_batch_strided
#define mkl_comatcopy_batch_strided MKL_Comatcopy_batch_strided
#define mkl_zomatcopy_batch_strided MKL_Zomatcopy_batch_strided
#define mkl_somatadd_batch_strided MKL_Somatadd_batch_strided
#define mkl_domatadd_batch_strided MKL_Domatadd_batch_strided
#define mkl_comatadd_batch_strided MKL_Comatadd_batch_strided
#define mkl_zomatadd_batch_strided MKL_Zomatadd_batch_strided
#define mkl_simatcopy_batch MKL_Simatcopy_batch
#define mkl_dimatcopy_batch MKL_Dimatcopy_batch
#define mkl_cimatcopy_batch MKL_Cimatcopy_batch
#define mkl_zimatcopy_batch MKL_Zimatcopy_batch
#define mkl_somatcopy_batch MKL_Somatcopy_batch
#define mkl_domatcopy_batch MKL_Domatcopy_batch
#define mkl_comatcopy_batch MKL_Comatcopy_batch
#define mkl_zomatcopy_batch MKL_Zomatcopy_batch
