/*******************************************************************************
* Copyright (C) 2019 Intel Corporation
*
* This software and the related documents are Intel copyrighted  materials,  and
* your use of  them is  governed by the  express license  under which  they were
* provided to you (License).  Unless the License provides otherwise, you may not
* use, modify, copy, publish, distribute,  disclose or transmit this software or
* the related documents without Intel's prior written permission.
*
* This software and the related documents  are provided as  is,  with no express
* or implied  warranties,  other  than those  that are  expressly stated  in the
* License.
*******************************************************************************/

/*
!  Content:
!      Intel(R) oneAPI Math Kernel Library (oneMKL) C interface for
!      OpenMP offload
!******************************************************************************/

#ifndef _MKL_OMP_OFFLOAD_H_
#define _MKL_OMP_OFFLOAD_H_

#include "mkl_blas_omp_offload.h"
#include "mkl_lapack_omp_offload.h"
#include "mkl_dfti_omp_offload.h"
#include "mkl_vsl_omp_offload.h"
#include "mkl_vml_omp_offload.h"
#include "mkl_spblas_omp_offload.h"

#endif
